# `Skypager.Assets.Asset`

## The basis for our source code entity database 

In a Skypager Project there will be bunch of files with different extensions, different parsers, etc.  Javascript modules, Markdown Documents, YAML files that capture configuration, SVG Diagrams that get displayed, Stylesheets, etc.

Generally speaking, the Assets in a project will be "unique" to that Project, or a dependency of some sort. 

The `Skypager.Asset` class is used as the base template for the different abstract representations of all of these files and dependencies.  The `Skypager.Project` creates different `Skypager.Collection` instances to be in charge of each of the different types of `Skypager.Asset` subclasses.

This gives us a way of working with our files as data, which gives us additional ways of manipulating these files as our projects mature and evolve.

## Setup

```javascript
const Skypager = global.skypager
const Assets = Skypager.Assets
const project = global.project
```

## Specifications

### There are many different subclasses of `Skypager.Asset`

Generally speaking the different types break down by file extension, and each class will define different types of parsers, indexers, and transformer functions.  

Some assets are just there.  `Assets.Image` for example.  Other assets such as `Assets.Document` or `Assets.HelperScript` play a much more critical role

```javascript
Assets.should.have.property('Asset')
Assets.should.have.property('CopyFile')
Assets.should.have.property('DataSource')
Assets.should.have.property('Document')
Assets.should.have.property('HelperScript')
Assets.should.have.property('Image')
Assets.should.have.property('ProjectManifest')
Assets.should.have.property('Script')
Assets.should.have.property('SettingsFile')
Assets.should.have.property('StyleSheet')
Assets.should.have.property('Vector')
```

### Each subclass will define its own `GLOB` and `EXTENSIONS` constants

These patterns are used to assign the correct files to the `Skypager.Project` content collections.

```javascript
Assets.CopyFile.should.have.property('GLOB')
Assets.DataSource.should.have.property('GLOB')
Assets.Document.should.have.property('GLOB')
Assets.Image.should.have.property('GLOB')
Assets.HelperScript.should.have.property('GLOB')
Assets.ProjectManifest.should.have.property('GLOB')
Assets.Script.should.have.property('GLOB')
Assets.Stylesheet.should.have.property('GLOB')
Assets.SettingsFile.should.have.property('GLOB')

Assets.Vector.should.have.property('EXTENSIONS')
Assets.CopyFile.should.have.property('EXTENSIONS')
Assets.DataSource.should.have.property('EXTENSIONS')
Assets.Document.should.have.property('EXTENSIONS')
Assets.Image.should.have.property('EXTENSIONS')
Assets.HelperScript.should.have.property('EXTENSIONS')
Assets.ProjectManifest.should.have.property('EXTENSIONS')
Assets.Script.should.have.property('EXTENSIONS')
Assets.Stylesheet.should.have.property('EXTENSIONS')
Assets.SettingsFile.should.have.property('EXTENSIONS')
Assets.Vector.should.have.property('EXTENSIONS')
```

### The `Skypager.Project` assigns `Skypager.Asset` subclasses to each collection

Generally this is because each type of asset will need a different parser, but it also encompasses common use cases and scenarios for working with certain types of assets in a `Skypager.Project`

```javascript
project.collections.forEach(collection => {
  collection.should.have.property('AssetClass').that.is.a('function')
})
```

### The `Skypager.Asset` class can customize a collection's interface

The `Assets.HelperScript` for example, creates query helpers for the different categories of helper scripts it expects to find.

```javascript
Assets.HelperScript.should.have.property('customizeCollection')
project.content.helpers.should.have.property('actions')
project.content.helpers.should.have.property('importers')
project.content.helpers.should.have.property('exporters')
```
