'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _arguments = arguments;

var assetLoader = (function () {
  var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    for (var _len = _arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = _arguments[_key];
    }

    var _require, readFile, options, asset, callback, raw;

    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _require = require('fs-promise');
            readFile = _require.readFile;
            options = args.find(function (o) {
              return o && o.asset;
            });
            asset = options && options.asset;
            callback = args.find(function (o) {
              return typeof o === 'function';
            });

            if (asset) {
              _context.next = 7;
              break;
            }

            return _context.abrupt('return');

          case 7:
            _context.prev = 7;
            _context.next = 10;
            return readFile(asset.paths.absolute).then(function (buffer) {
              return buffer.toString();
            });

          case 10:
            raw = _context.sent;

            asset.raw = raw;
            asset.assetWasImported();
            _context.next = 19;
            break;

          case 15:
            _context.prev = 15;
            _context.t0 = _context['catch'](7);

            if (asset) {
              asset.error = _context.t0;
            }
            throw _context.t0;

          case 19:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[7, 15]]);
  }));
  return function assetLoader() {
    return ref.apply(this, arguments);
  };
})();

exports.DiskImporter = DiskImporter;
exports.AssetImporter = AssetImporter;
exports.ProjectImporter = ProjectImporter;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
* A non-blocking script which consolidates all interaction with the file system.
*
* The thinking behind separating this was inspired by the desire to have the skypager
* library also work in the browser, but have swappable backends for asset and data source
* content.
*/
function DiskImporter() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var context = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var project = options.project = options.project || this;

  if (options.asset && options.collection) {
    return AssetImporter.apply(project, arguments);
  }

  if (options.project) {
    return ProjectImporter.apply(project, arguments);
  }
}

/*
* This assumes an asset is in the collection already.

*/
function AssetImporter() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var context = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var asset = options.asset;

  if (!asset) {
    console.log('Asset Importer Called', options, context);
  }

  var callback = options.onComplete;

  if (typeof context === 'function') {
    callback = context;
  }

  if (options.sync || process.env.SKYPAGER_SYNC_IMPORTERS) {
    asset.raw = require('fs').readFileSync(asset.paths.absolute).toString();
    callback && callback(this);
  } else if (asset) {
    assetLoader(options, callback);
  }

  return asset;
}

function ProjectImporter() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
  var context = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var path = require('path');
  var project = this;
  var collections = options.collections || project.content;
  var autoLoad = options.autoLoad || {};

  (0, _keys2.default)(collections).forEach(function (name) {
    var collection = collections[name];
    collection.loadAssetsFromDisk();
  });

  var callback = options.onComplete;

  callback && callback(project, options);

  return project;
}

exports.default = DiskImporter;