'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.logger = logger;

var _winston = require('winston');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function logger(project) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  if (project.logger) {
    return project.logger;
  }

  var debug = require('yargs').argv.debug || false;
  var env = project.env;

  project.ensurePath('logs');

  var _transports = [new _winston.transports.File({
    name: 'project',
    filename: project.path('logs', 'project.' + project.env + '.log'),
    timestamp: true,
    json: false,
    prettyPrint: false,
    maxSize: 1024
  })];

  if (process.env.SKYPAGER_PRETTY_LOGGING) {
    _transports.push(new _winston.transports.File({
      name: 'project-pretty',
      filename: project.path('logs', 'pretty.' + project.env + '.log'),
      timestamp: false,
      json: false,
      prettyPrint: true,
      colorize: true
    }));
  }

  if (options.enableConsole || process.env.SKYPAGER_ENABLE_CONSOLE_LOG === true) {
    _transports.push(new _winston.transports.Console({
      name: 'console',
      colorize: true,
      prettyPrint: true,
      level: debug ? 'debug' : 'info'
    }));
  }

  defineProperty(project, 'logger', {
    enumerable: false,
    configurable: false,
    value: new _winston.Logger({
      level: 'debug',
      transports: _transports
    })
  });

  assign(project, {
    //TODO: define metadata

    _log: function _log(level) {
      var _project$logger;

      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      (_project$logger = project.logger).log.apply(_project$logger, [level].concat((0, _toConsumableArray3.default)(args)));
    },
    debug: function debug() {
      for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      project._log.apply(project, ['debug'].concat((0, _toConsumableArray3.default)(args)));
    },
    info: function info() {
      for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        args[_key3] = arguments[_key3];
      }

      project._log.apply(project, ['info'].concat((0, _toConsumableArray3.default)(args)));
    },
    log: function log() {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      project._log.apply(project, ['info'].concat((0, _toConsumableArray3.default)(args)));
    },
    warn: function warn() {
      for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
        args[_key5] = arguments[_key5];
      }

      project._log.apply(project, ['warn'].concat((0, _toConsumableArray3.default)(args)));
    },
    error: function error() {
      for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        args[_key6] = arguments[_key6];
      }

      project._log.apply(project, ['error'].concat((0, _toConsumableArray3.default)(args)));
    }
  });

  return project.logger;
}

exports.default = logger;
var _Object = Object;
var assign = _Object.assign;
var defineProperty = _Object.defineProperty;