import { transform } from 'babel-core'
import { parse } from 'babylon'

export function parser(code='', asset) {
  if(!code || code.length === 0) {
    asset.project.warn('attempt to parse empty asset: ' + asset.id)
  }

  try {
    return parse(code, asset.parserOptions)
  } catch(error) {
    asset.project.error('Parser Error in Asset: ' + asset.id, error.message)
  }
}

export function compiler(code, asset) {
  return transformer(code, asset).code
}

export function transformer(code, asset) {
  const id = asset.id

  try {
    return transform(
      code,
      asset.transformerOptions
    )
  } catch(error) {
    return {
      ast: {},
      code: `console.log('Error while compiling ${ id }', '${error.message}')`,
      map: ``
    }
  }
}

function attemptRequire(moduleId) {
  try {
    return require(moduleId)
  } catch(error) {}
}
