import DataSource from './data_source'

import get from 'lodash/get'
import clone from 'lodash/clone'
import omit from 'lodash/omit'
import omitBy from 'lodash/omitBy'
import result from 'lodash/result'
import defaults from 'lodash/defaultsDeep'

const EXTENSIONS = ['js', 'json', 'yaml', 'yml', 'csv']
const GLOB = '**/*.{' + EXTENSIONS.join(',') + '}'

export class SettingsFile extends DataSource {

  static EXTENSIONS = EXTENSIONS;
  static GLOB = GLOB;

  static decorateCollection(collection, options = {}) {
    defineProperty(collection, 'notSecret', {
       enumerable: true,
       configurable: false,
       get: function(){
         return collection.where(file => {
            return !file.id.match(/(secret|secure|private|hidden)/i)
         })
       }
    })
  }

  constructor (uri, options = {}) {
    super(uri, options)

    this.indexer = parseSettings.bind(this)
  }

  /*
   * This lets us write less nested
   * versions of a settings file
   */
  getData() {
    if (this.extension !== '.js' && (!this.raw || this.raw.length === 0)) {
      this.runImporter('disk', {asset: this, sync: true})
    }

    let value = this.indexed
    let parts = this.id.split('/')
    let last = parts[ parts.length - 1]

    /**
     * If the settings yaml has a single key whose value
     * is the name of the settings file, we will use the value
     * at that key instead of the redundant nesting
     */
    if (value[last] && keys(value).length === 1) {
      value = value[last] || {}
    }

    if (get(value, this.project.env)) {
      value = get(value, this.project.env)
    }

    let settingsDefaults = (last !== 'defaults' && get(this.indexed, 'defaults', {}))

    value = omit(value, 'defaults')

    let data = settingsDefaults
      ? defaults({}, value, settingsDefaults)
      : value

    return omitBy(data, (v,k) => k.match(/(secure|secret|private|hidden)/i))
  }
}

export default SettingsFile

function interpolateValues (obj, template) {
  let datasource = this

  Object.keys(obj)
  .filter(key => !key.match(/(secret|secure|private|hidden)/i))
  .forEach(key => {
    let value = obj[key]

    if (typeof value === 'object') {
      interpolateValues.call(datasource, value, template)
    } else if (typeof value === 'string' && value.match(/^env\.\w+$/i)) {
      value = `<%= process.${ value } %>`
      obj[key] = template(value)(value)
    } else if (typeof value === 'string' ) {
      obj[key] = template(value)(value)
    }
  })

  return obj
}

function parseSettings (val = {}) {
  let base = interpolateValues.call(this, clone(val), this.templater.bind(this))

  return omitBy(base, (v,k) => {
     return k.match(/(secret|secure|hidden|private)/i)
  })
}

const { defineProperty, keys } = Object
